/*

  PONGDuino : a Pong game for Arduino and 16x2 LCD display

  Copyright (c) 2013, Xavier Miller (xavier.miller@cauwe.org)
  http://www.xaviermiller.be

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  - Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef __LCD_PONG_H__
#define __LCD_PONG_H__

//#include "pong.h"
//#include <LiquidCrystal.h>

// http://arduino.cc/en/uploads/Tutorial/LCD_bb.png

class LCDPong : public LiquidCrystal, public IPongReceiver
{
private:
    Pong *m_pong;
    bool m_running;

    // IPongReceiver
    void Pause();
    void Continue();

    void DisplayScore(int score1, int score2);
    void PauseScreen(int score1, int score2, const unsigned char *message);
    void GameScreen(int score1, int score2, int ball_x, int ball_y, int ball_size, int paddle1_x, int paddle1_y, int paddle2_x, int paddle2_y, int paddle_width, int paddle_height);

    void BeepWall() {} // TODO
    void BeepPaddle() {} // TODO
    void BeepLost() {} // TODO
public:
    LCDPong();
    ~LCDPong();
    
public:
    void keyPressPause() { m_pong->KeyPause(); }
    void keyPressUp1() { m_pong->KeyUp(Pong::Player1); }
    void keyPressDown1() { m_pong->KeyDown(Pong::Player1); }
    void keyPressUp2() { m_pong->KeyUp(Pong::Player2); }
    void keyPressDown2() { m_pong->KeyDown(Pong::Player2); }

    void timerEvent();
    const bool isRunning() const { return m_running; }
};

#endif
