/*

  PONGDuino : a Pong game for Arduino and 16x2 LCD display

  Copyright (c) 2013, Xavier Miller (xavier.miller@cauwe.org)
  http://www.xaviermiller.be

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  - Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef PONG_H
#define PONG_H

class IPongReceiver
{
public:
    virtual void Pause() = 0;
    virtual void Continue() = 0;

    virtual void PauseScreen(int score1, int score2, const unsigned char *message) = 0;
    virtual void GameScreen(int score1, int score2, int ball_x, int ball_y, int ball_size, int paddle1_x, int paddle1_y, int paddle2_x, int paddle2_y, int paddle_width, int paddle_height) = 0;

    virtual void BeepWall() = 0;
    virtual void BeepPaddle() = 0;
    virtual void BeepLost() = 0;
};

class Pong
{
private:
    Pong();
public:
    Pong(IPongReceiver *receiver, int width, int height, int paddle_size = 6, int paddle_width = 2, int ball_size = 2, int dir_col = 2, int dir_line = 1, int score_max = 99);

    enum PongPlayer { Player1, Player2 };

    void Reset();

    void KeyPause();
    void KeyUp(PongPlayer player);
    void KeyDown(PongPlayer player);

    void TimerTick();
private:
    enum PongPadDirection { Up, Down };
    enum PongStatus { Paused, Running };
    enum PongPauseType { PauseSplash, PauseDefault, PausePlayer1Win, PausePlayer2Win, GameOver };

    void CenterPaddles();
    void CenterBall();
    void NewBall();

    void MovePlayer(PongPlayer player, PongPadDirection direction);
    void Draw();

private:
    IPongReceiver *m_receiver;

    int m_width;
    int m_height;

    int m_col;
    int m_line;
    int m_paddle_size;
    int m_paddle_width;
    int m_ball_size;
    int m_dir_col;
    int m_dir_line;
    int m_left_paddle;
    int m_right_paddle;
    int m_score1;
    int m_score2;
    int m_score_max;

    PongStatus m_status;
    PongPauseType m_pause_type;
};

#endif // PONG_H
