/*

  PONGDuino : a Pong game for Arduino and 16x2 LCD display

  Copyright (c) 2013, Xavier Miller (xavier.miller@cauwe.org)
  http://www.xaviermiller.be

  All rights reserved.

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

  - Redistributions of source code must retain the above copyright
  notice, this list of conditions and the following disclaimer.

  - Redistributions in binary form must reproduce the above copyright
  notice, this list of conditions and the following disclaimer in the
  documentation and/or other materials provided with the distribution.
  
  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
  COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
  ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
  POSSIBILITY OF SUCH DAMAGE.

*/

#include "pong.h"
#include "LiquidCrystal.h"
#include "lcdpong.h"

// LCD 16x2 setup tutorial: http://arduino.cc/en/uploads/Tutorial/LCD_bb.png

#define KEY_COUNT 5

#define PAUSE_BUTTON  10
#define LEFT_1 6
#define RIGHT_1 7
#define LEFT_2 8
#define RIGHT_2 9


/** Broche pour le contrôle du rétroéclairage */
const byte BACKLIGHT_PWM_PIN = 10;

/** Énumération des boutons utilisables */
enum {
  BUTTON_NONE,  /*!< Pas de bouton appuyé */
  BUTTON_UP,    /*!< Bouton UP (haut) */
  BUTTON_DOWN,  /*!< Bouton DOWN (bas) */
  BUTTON_LEFT,  /*!< Bouton LEFT (gauche) */
  BUTTON_RIGHT, /*!< Bouton RIGHT (droite) */
  BUTTON_SELECT /*!< Bouton SELECT */
};

/** Retourne le bouton appuyé (si il y en a un) */
byte getPressedButton() {

  /* Lit l'état des boutons */
  int value = analogRead(A0);

  /* Calcul l'état des boutons */
  if (value < 50)
    return BUTTON_RIGHT;
  else if (value < 250)
    return BUTTON_UP;
  else if (value < 450)
    return BUTTON_DOWN;
  else if (value < 650)
    return BUTTON_LEFT;
  else if (value < 850)
    return BUTTON_SELECT;
  else
    return BUTTON_NONE;
}


  
LCDPong *lcdpong;

byte buttons[KEY_COUNT] = { PAUSE_BUTTON, LEFT_1, RIGHT_1, LEFT_2, RIGHT_2 };
int buttonState[KEY_COUNT] = { LOW, LOW, LOW, LOW, LOW };
long lastDebounceTime[KEY_COUNT] = { 0, 0, 0, 0, 0 };
byte keyWasDown[KEY_COUNT] = { 1, 1, 1, 1, 1 };


long lastTimer = 0;

#define DEBOUNCE_DELAY 50


int LectureBouton(){
switch (getPressedButton()) {
  case BUTTON_NONE:
    
    break;

  case BUTTON_UP:
    buttonState[3]=HIGH;
    Serial.println("R2");
    break;

  case BUTTON_DOWN:
    buttonState[4]=HIGH;
    Serial.println("L2");
    break;

  case BUTTON_LEFT:
    buttonState[2]=HIGH;
    Serial.println("R1");
    break;
    
  case BUTTON_RIGHT:
    buttonState[0]=HIGH;
    Serial.println("Pause");
    break;

  case BUTTON_SELECT:
    buttonState[1]=HIGH;
    Serial.println("L1");
    break;
  }
  delay(DEBOUNCE_DELAY);
}


void setup()
{
  Serial.begin(9600);
  pinMode(BACKLIGHT_PWM_PIN, OUTPUT);
  //analogWrite(BACKLIGHT_PWM_PIN, 0); // Pas de rétroéclairage
  analogWrite(BACKLIGHT_PWM_PIN, 50);
  // analogWrite(BACKLIGHT_PWM_PIN, 255); // Rétro-éclairage 100%
  for (int key = 0; key < KEY_COUNT; key++)
    pinMode(buttons[key], INPUT);
  Serial.println("Demarrage");
  lcdpong = new LCDPong();
  Serial.println("init terminee");
}

void loop()
{
  // read keys
  /* for (byte key = 0; key < KEY_COUNT; key++)
  {
    int reading = digitalRead(buttons[key]);
    if ((millis() - lastDebounceTime[key]) > DEBOUNCE_DELAY)
    {
      if (reading != buttonState[key])
      {
        Serial.print("State changed to ");
        Serial.println(reading);
        buttonState[key] = reading;
        lastDebounceTime[key] = millis();
      }
    }
    if (buttonState[key] == LOW)
      keyWasDown[key] = 1;
  } */
  buttonState[0]=LOW;
  buttonState[1]=LOW;
  buttonState[2]=LOW;
  buttonState[3]=LOW;
  buttonState[4]=LOW;
  for (byte key = 0; key < KEY_COUNT; key++) keyWasDown[key] = 1;
  LectureBouton();
  // Pause Key
  if ((buttonState[0] == HIGH) && keyWasDown[0])
  {
    keyWasDown[0] = 0;
    lcdpong->keyPressPause();
  }

  // Left 1
  if ((buttonState[1] == HIGH) && keyWasDown[1])
  {
    keyWasDown[1] = 0;
    lcdpong->keyPressUp1();
  }
  // Right 1
  if ((buttonState[2] == HIGH) && keyWasDown[2])
  {
    keyWasDown[2] = 0;
    lcdpong->keyPressDown1();
  }

  // Left 2
  if ((buttonState[3] == HIGH) && keyWasDown[3])
  {
    keyWasDown[3] = 0;
    lcdpong->keyPressUp2();
  }
  // Right 2
  if ((buttonState[4] == HIGH) && keyWasDown[4])
  {
    keyWasDown[4] = 0;
    lcdpong->keyPressDown2();
  }

  if (lcdpong->isRunning())
  {
    if ((millis() - lastTimer) > 100)
    {  
      lcdpong->timerEvent();
      lastTimer = millis();
    }
  }
}
